// This program will slice an image into boxes according to the number
// of rows and columns specified. This is done for the purpose of creating
// ROIs for Fulvio's enhanced PTViewer.
//
// This program is compatible with PTViewer v2.7L2 or higher.
// The *.ptv and *.ptvref output files are only compatible with 
// PTViewer v2.8 beta3 or higher.
//
// By Eric Gerds v1.3                     http://www.pinlady.net/vr
// Compatible with Photoshop CS/CS2
//
// Photoshop CS2 bug: The script listener code to erase all image slices
// does not work (but does work in PS CS). Thus, we will disable the drawing
// of image slices in CS2.


// Define the global parameters
function globalparams(){
if (app.documents.length==0){return(false)};

// The name and version of this script
G.name = "ImageSlicerPT";
G.version = G.name + " v1.3";

G.PSversion = PSversion();

// Preferences file
G.fileprefs = new File(app.path.toString() + "/ImageSlicerprefs.txt");

getnewline(); // Define newline depending on your OS

G.docref = app.activeDocument;

// Width and Height must be rounded due to an occaisonal Photoshop bug.
// Sometimes the results are not integer, but have a very small error
// in them (~ 1e-13).
G.docwidth = Math.round(G.docref.width.value); // document width
G.docheight = Math.round(G.docref.height.value); // document height

G.cx = (G.docwidth - 1)/2; // center x of doc
G.cy = (G.docheight - 1)/2; // center y of doc

// Macintosh file creator, file type.
// It is not clear what values to use when reading in .jpg as
// a binary file, or when creating the .html, .ptv, or .ptvref files.
// These vars are needed by the 'open()' method on a MAC.
G.MACtype = "TEXT";
G.MACcreator = "ttxt";


// Arrays to hold the X and Y boundaries of the image grid,
// which is used to create the image slices
G.gridX = new Array();
G.gridY = new Array();

// Arrays to hold the jpg quality of each grid element.
// Quality of grid element X,Y is G.qualX + G.qualY
G.qualXY = new Array();

// Maximum compression values allowed for jpgs
G.maxcompsaveforweb = 100; // Save for web allows 0 - 100 compression
G.maxcomp = 12;       // Ordinary save allows 0 - 12 compression

// defaults
G.rowsdef = 1;        // Number of rows to slice image
G.columnsdef = 4;     // Number of columns to slice image
G.qualitydef = 40;    // Jpg quality of image slices
G.customdef = false;  // Specify quality for each slice individually?
G.prefixdef = '';      // Prefix for output file names
G.folderdef = '';      // Path to folder with images
G.previewdef = true;   // Generate preview image?
G.prevgreydef = true;  // Preview image greyscale?
G.prevwidthdef = 100;  // Preview image width
G.prevqualdef = 40;    // Preview image jpeg quality
G.diswidthdef = 700;   // Display width
G.disheightdef = 500;  // Display height
G.showslicesdef = false;

// These 3 params are for program capabilities that are now hardwired
G.savewebdef = true; // Use SaveForWeb to create jpgs?
G.puredef = false;  // Run pure jpeg?
G.purelocdef = '';  // Location of pure jpeg program


return true;
}

// Return Photoshop version.
// app.version is a string of the form 8.0.0
// Just in case, I also check to see if it is of the form 8,0,0
// Photoshop CS = version 8
// Photoshop CS2 = version 9
function PSversion(){
var version = app.version; // string
period=version.indexOf(".",0)

if (period==-1){period=version.indexOf(",",0)}

// Convert the string to a number using decimal()
if (period>-1){return( decimal(version.substring(0,period)) )}
else{return decimal(version)};

}


function getnewline(){
// 0 for Windows, 1 for Mac, 2 for Unix
G.OS = YourOS();

// Determining text file NewLine depends on the OS you are using.
// Windows sees a newline as CarriageReturn(CR) + LineFeed(LF).
// Macintosh sees a newline as CR.
// Unix sees a newline as LF.
// Use the G.newline variable whenever you need a new line in an
// edittext field in a dialog.
if (G.OS==0){G.newline="\u000D\u000A"}; // Windows
if (G.OS==1){G.newline="\u000D"};       // Macintosh
if (G.OS==2){G.newline="\u000A"};       // Unix

}

// Save photoshop state
function savestate(){
if (app.documents.length==0){return(false)};

// Save displayDialogs value
G.startdisplayDialogs = app.displayDialogs;

// Turn off all the dialogs
app.displayDialogs = DialogModes.NO;

G.strtRulerUnits = app.preferences.rulerUnits;
G.strtTypeUnits = app.preferences.typeUnits;
G.bgColor = app.backgroundColor;
G.fgColor = app.foregroundColor;

//  Set units make sure they are PIXELS
app.preferences.rulerUnits = Units.PIXELS;
app.preferences.typeUnits = TypeUnits.PIXELS;

return true;
}

// Restore photoshop state
function restorestate(){
if (app.documents.length==0){return(false)};

// Restore Photoshop Preferences
app.preferences.rulerUnits = G.strtRulerUnits;
app.preferences.typeUnits = G.strtTypeUnits;
app.backgroundColor = G.bgColor;
app.foregroundColor = G.fgColor;

// Restore dialogs
app.displayDialogs = G.startdisplayDialogs;

}

// Open a dialog, have the user select a folder
// Return the folder object.
// To get the folder path, use P.fsName
function getfolder(){
var A = new Folder("~");

var P = Folder.selectDialog(
   "Choose a folder to save the image slices in",A);

if (P) {return(P) }   // Was a Folder Chosen?
else {return(null)};       // No folder was chosen

}

// Get purejpeg file.
//
// Note: This routine is no longer used because Purejpeg is no longer
// used.
function getpurefile(){

var P = File.openDialog("Location of purejpeg.exe",
   "exe:*.exe");
if (P){return(P)}
else {return(null)};

}


// Apparently it is not entirely safe to convert a string to a number
// in Photoshop using string*1 when the string contains floating point numbers.
// This can give errors in some international Photoshop versions.
// Hence we must use a roundabout method of converting strings to floating
// point numbers.
// Many thanks to Erik Krause for hunting down this tricky bug and for
// supplying the ideas/code to solve this problem.
//
// This function will convert both "5.5" and "5,5" to a number.
//
// A non numeral in the string will return 0.
function decimal(a){
if (typeof(a)=="number"){return(a)};
if (typeof(a)!="string"){return(0)};

a = a.replace(/\s/g,""); // replace blanks in the string
if (a==""){return(0)};

// test if a numerical digit is present 1 or more times. If not then
// return 0.
var pattern=/[0-9]+/;  // test for 0-9 digits being present 1 or more times
if (pattern.test(a)==false){return(0)};

var b=[],c=[],pw=0,exp=1,k0,k1;

c[0] = a;
// Consider numbers like 5.4e-005, 5.4e+004 = numbers with exponent
if (a.indexOf("e-")>-1){c=a.split("e-");pw=-1};
if (a.indexOf("E-")>-1){c=a.split("E-");pw=-1};
if (a.indexOf("e+")>-1){c=a.split("e+");pw=+1};
if (a.indexOf("E+")>-1){c=a.split("E+");pw=+1};
if (pw!=0){
  try{exp=Math.pow(10, pw*eval((c[1]=="")?"0":c[1]))}
  catch(e){return(0)} // Error, not a number, so return 0
  };        // Note, eval("")=undefined


a = c[0];
//   get minus sign if present, gets lost if -0 is before dot
var sign = (a.indexOf("-") > -1) ? -1 : 1;

// Is 'a' an integer?
if ((a.indexOf(".")<0)&&(a.indexOf(",")<0)){b[0]=a;b[1]=""};

// Split number string at the dot or comma, 12.34 into 12 and 34
if ((a.indexOf(".")>-1)&&(a.indexOf(",")<0)){b=a.split(".")};
if ((a.indexOf(".")<0)&&(a.indexOf(",")>-1)){b=a.split(",")};


// eval("")=undefined, eval(""/1)=0)""/1). 
// I don't trust the reliability of this, so I explicitly test for "".
if (b[1]==""){k1=0}
else{
  try{k1=eval(b[1]/Math.pow(10, b[1].length))}
  catch(e){return(0)} // Error, not a number, so return 0 - or return NaN?
   };

// This takes care of "+.5" "-.6" ".7" also
if ((b[0]=="")||(b[0]=="+")||(b[0]=="-")){k0=0}
else{
  try{k0=eval(b[0])}
  catch(e){return(0)} // Error, not a number, so return 0 - or return NaN?
   };

// add the values again using calculated fraction, and multiply exponent
a = (k0 + k1)*exp;

// restore sign (because eval(-0) is 0;
a = Math.abs(a) * sign;

return(a);
}

// Make sure each grid box has a valid text value.
// Should be an integer from 0 to max for jpeg comppression.
// Max will be = G.maxcomp or G.maxcompsaveforweb, depending on whether the 
// Save For Web option is chosen.
function checkgridxy(itemxy,max){
var tmp;
tmp = decimal(itemxy.text);
if (tmp<0) {tmp = 0};
if (tmp>max) {tmp = max};

tmp = Math.round(tmp);
itemxy.text = tmp.toString();

}

// Determine which OS you are using
// Return '0' if Windows, '1' if Macintosh, '2' if Unix
// Or, if not Windows or Mac, then just return 1 by default?
function YourOS(){
var OS = File.fs;
OS = OS.charAt(0);

// We check only the first letter, both for small and capital
if ((OS=="W")||(OS=="w")){return(0)};  // Windows
if ((OS=="M")||(OS=="m")){return(1)};  // Macintosh
if ((OS=="U")||(OS=="u")){return(2)};  // Unix

}


function qualityhelp(maxquality){
var winheight = 100;

var about = new Window('dialog', 'About',[10,10,10 + 370,10 + winheight]);
about.center();

about.msg = about.add('statictext',[10,5,360,winheight-50],'',
  {multiline:true});

about.bye = about.add('button',[145,winheight-40,145+80,winheight-40+30],'OK');
about.bye.onClick = function(){about.close()};

var mess =
 "You may set the jpeg compression levels for each individual slice. " +
 "Enter any integer values from 0 to " + maxquality + ".";

about.msg.text = mess;

about.show();

}

// Gui to set jpeg quality for each grid box individually
function qualitygui(rows,columns,quality,maxquality){
var x,y; // loop counters

// Spacing from one grid box to the next
var horzspace = 30, vertspace = 25;

// We calculate the size of this gui window, based on the number
// of rows and columns. But, we add some extra space for buttons
// to winheight.
var winwidth = columns * horzspace + 20,
    winheight = rows * vertspace + 100;
if (winwidth<180){winwidth=180};

var Qdlg = new Window('dialog', 'Jpeg Slices',
   [10,30,10 + winwidth,30 + winheight]);

var top = 40, left = winwidth/2 - columns * horzspace/2;

for (y = 0;y<rows;y++){
  for (x = 0; x<columns; x++){
     eval("Qdlg.item" + x.toString() + y.toString() +
       " = Qdlg.add(\'edittext\',[" +
       (left+horzspace*x) + "," + (top+vertspace*y) + "," +
       (left+horzspace*x + horzspace - 5) +
      "," + (top+vertspace*y + vertspace-5) +
       "],G.qualXY[y][x].toString() );");

     eval("Qdlg.item" + x.toString() + y.toString() + ".onChange = " +
       " function(){checkgridxy(Qdlg.item" + x.toString() + y.toString() +
       "," + maxquality.toString() + ")};");
     }
   };

var ok = 0;

// Ok button
var but1y = winheight - 40;
var but1w = 60;
var but1x = winwidth/2  - but1w -5;
Qdlg.accept = Qdlg.add('button',[but1x,but1y,but1x+but1w,but1y+30],
   'Ok');
Qdlg.accept.onClick = function(){ok = 1;Qdlg.close()};

// Cancel button
Qdlg.bye = Qdlg.add('button',[10+but1x+but1w,but1y,10+but1x+2*but1w,but1y+30],
   'Cancel');
Qdlg.bye.onClick = function(){ok = 0;Qdlg.close()};

// Exit, help buttons
Qdlg.byebye = Qdlg.add('button',[winwidth-30,1,winwidth-30+25,1+25],'X');
Qdlg.byebye.onClick = function(){ok = 0;Qdlg.close()};

Qdlg.help = Qdlg.add('button',[winwidth-60,1,winwidth-60+25,1+25],'?');
Qdlg.help.onClick = function(){qualityhelp(maxquality)};

Qdlg.show();


// If you press the ok button, then put each grid number into
// a 2 dimensional array G.qualXY
//
// We should add error checking logic here, to make sure that
// every grid entry is a valid number
// Values should be integer between 0 to maxquality for jpeg compression.
var tmp;
if (ok==1){
  for (y = 0;y<rows;y++){
     for (x = 0; x<columns; x++){
     eval("checkgridxy(Qdlg.item" + x.toString() + y.toString() +
     "," + maxquality.toString() + ")");
     G.qualXY[y][x] = decimal( eval("Qdlg.item" + x.toString() + y.toString() +
        ".text") );
         }
      }
    }


} // end of qualitygui()


function help(){
var winheight = 420;

var about = new Window('dialog', 'About ' + G.name,
  [10,10,10 + 370,10 + winheight]);
about.center();

about.msg = about.add('statictext',[10,5,360,winheight-50],'',
  {multiline:true});

about.bye = about.add('button',[145,winheight-40,145+80,winheight-40+30],'OK');
about.bye.onClick = function(){about.close()};

var mess =
 G.version + G.newline +
 "By Eric Gerds               http://www.pinlady.net/vr/" +
 G.newline + G.newline +
 G.name + " is a Photoshop CS/CS2 script that will slice up a panoramic " +
 "image for use with PTViewer (version 2.7L2 or HIGHER). " +
 "Each image slice is loaded into PTViewer as an ROI (region of interest), " +
 "meaning that your panorama image will be progressively loaded in pieces. " +
 G.newline + G.newline +
 "You can get the latest version of PTViewer from Fulvio Senore at " +
 "http://www.fsoft.it/panorama/ptviewer.htm" +
 G.newline + G.newline +
 "To set the jpg compression for each slice individually, select the " +
 "'Customized slices' option and then press the 'Set each slice' button. " +
 "A new window will pop up, allowing you to enter a compression value " +
 "for each of the image slices. Please note that when the " + 
 "'Customized slices' option is chosen, that the number of rows and " +
 "columns cannot be changed." +
 G.newline + G.newline +
 "The jpg slice compressions can " +
 "be set to any integer from 0 to 100. The slices will be optimized to " +
 "reduce file size." + G.newline + G.newline +

 "Note: The *.ptv and *.ptvref output files are only compatible with " +
 "PTViewer v2.8 beta3 or higher.";

about.msg.text = mess;

about.show();

}


// Main Gui window
function wingui(){
var aaa = readprefs(G.fileprefs);

// Note: Purejpeg is no longer used in this program
var purejpeg = new Array();
purejpeg[0] = aaa[10]; // boolean
purejpeg[1] = aaa[11]; // string

// The Saveforweb option will now be always true
var saveweb = true // = aaa[14];

var rows = aaa[0], columns = aaa[1], quality = aaa[2],
  custom = aaa[3], prefix = aaa[4], folder = aaa[5],
  preview = aaa[6], prevgrey = aaa[7], prevwidth = aaa[8],
  prevquality = aaa[9];

var displaywidth = aaa[12], displayheight = aaa[13], showslices = aaa[15];


// Define the window gui dimensions
var dlg = new Window('dialog', G.version,
   [10,10,10 + 370,10 + 600]);
dlg.center();

dlg.p1 = dlg.add('panel',[1,30,369,30+105],'Number of Image Slices');
dlg.p2 = dlg.add('panel',[1,140,369,140+90],'Quality of Image Slices')
dlg.p3 = dlg.add('panel',[1,230,369,230+100],'');
dlg.p4 = dlg.add('panel',[1,330,369,330+100],'');
dlg.p5 = dlg.add('panel',[1,435,369,435+90],'PTViewer Window Size');

// Display the memory use for PTViewer
dlg.mem = dlg.add('statictext',[5,5,200,30],'');

// ------------------- Panel 1 -------------------------------
var ready = true; // Variable used to make sure that Photoshop is ready
                  // to accept input to the gui sliders

var y1c = 15;
dlg.p1.grid = dlg.p1.add('checkbox',[5,y1c,120,y1c+25],'Show Slices');
dlg.p1.grid.onClick = function(){
  if (ready){
    ready = false;
    showslices = dlg.p1.grid.value;
     // erase slices if slice visibility was just turned off
    if (!showslices){eraseallslices()};
    creategrid(rows,columns,showslices);
    ready = true;
   }
  else{dlg.p1.grid.value = showslices};

};


var y1a = 45;
dlg.p1.m1 = dlg.p1.add('statictext',[5,y1a,70,y1a+20],
 'Rows');
dlg.p1.rows = dlg.p1.add('slider',[80,y1a,310,y1a+20],
    1,1,11);
dlg.p1.rowst = dlg.p1.add('edittext',[320,y1a-2,320+40,y1a+18],'');
dlg.p1.rows.onChange = function(){
  if (ready){
    ready = false;
    rows = Math.round(dlg.p1.rows.value);
    dlg.p1.rowst.text = rows;
    creategrid(rows,columns,dlg.p1.grid.value);
    dlg.mem.text = javamemoryuse(dlg);
    ready = true;
    }
 else{
    dlg.p1.rows.value = rows;
    dlg.p1.rowst.text = rows;
   };
};

var y1b = 75;
dlg.p1.m2 = dlg.p1.add('statictext',[5,y1b,70,y1b+20],
 'Columns');
dlg.p1.columns = dlg.p1.add('slider',[80,y1b,310,y1b+20],
    1,1,11);
dlg.p1.columnst = dlg.p1.add('edittext',[320,y1b-2,320+40,y1b+18],'');
dlg.p1.columns.onChange = function(){
  if (ready){
    ready = false;
    columns = Math.round(dlg.p1.columns.value);
    dlg.p1.columnst.text = columns;
    creategrid(rows,columns,dlg.p1.grid.value);
    dlg.mem.text = javamemoryuse(dlg);
    ready = true;
    }
  else{
    dlg.p1.columns.value = columns;
    dlg.p1.columnst.text = columns;
    };

};

// -------------------------- Panel 2 ---------------------------
var tmp2a, tmp2b, tmp2c, tmp2d, tmp2e, tmp2f;
var y2a = 25;
dlg.p2.m1 = dlg.p2.add('statictext',[5,y2a,85,y2a+20],
 'Jpeg Quality');
dlg.p2.quality = dlg.p2.add('slider',[90,y2a,310,y2a+20],
    4,0,(saveweb?G.maxcompsaveforweb:G.maxcomp));
dlg.p2.qualityt = dlg.p2.add('edittext',[320,y2a-2,320+40,y2a+18],'');
dlg.p2.quality.onChange = function(){dlg.p2.qualityt.text =
  Math.round(dlg.p2.quality.value)
  };
  
var y2b = 55;
var x1,y1;

dlg.p2.custom = dlg.p2.add('checkbox',[10,y2b,10+170,y2b+25],
   'Customized slices');
var x,y;
dlg.p2.custom.onClick = function(){
   tmp2a = dlg.p2.custom.value;
   tmp2b = true;
   if (tmp2a==false){
      tmp2b=confirm("Turning off this checkbox will erase \n" +
       "the customized compression values for\n" +
       "each jpeg slice. " + "Proceed?")};
   if (!tmp2b){dlg.p2.custom.value = true; return};

   dlg.p2.quality.enabled = !tmp2a;
   dlg.p2.qualityt.enabled = !tmp2a;
   dlg.p2.grid.enabled = tmp2a;
   
   dlg.p1.rows.enabled = !tmp2a;
   dlg.p1.rowst.enabled = !tmp2a;
   dlg.p1.columns.enabled = !tmp2a;
   dlg.p1.columnst.enabled = !tmp2a;
   
// If the custom checkbox is turned on, then initialize the G.qualXY
// array to contain the setting from the quality slider (for ALL array
// entries).
   if (tmp2a==true){
     G.qualXY = []; // empty the array
     for (y = 0;y<Math.round(dlg.p1.rows.value);y++){
       G.qualXY[y] = new Array();
       for (x = 0; x<Math.round(dlg.p1.columns.value); x++){
          G.qualXY[y][x] = Math.round(dlg.p2.quality.value)};
                          };
   
   }
}

dlg.p2.grid = dlg.p2.add('button',[200,y2b,200+130,y2b + 25],
    'Set each slice');
dlg.p2.grid.onClick = function(){
  qualitygui(Math.round(dlg.p1.rows.value),
      Math.round(dlg.p1.columns.value),
      Math.round(dlg.p2.quality.value),
      dlg.p2.quality.maxvalue
      );
};

// -------------------------- Panel 3 ---------------------------
var tmp3;
var y3a = 10;
dlg.p3.m0 = dlg.p3.add('statictext',[5,y3a,160,y3a+20],
  'Prefix for Image Slices:');
dlg.p3.prefix = dlg.p3.add('edittext', [165,y3a,360,y3a+20],'');

var y3b = 40;
dlg.p3.m1 = dlg.p3.add('statictext',[5,y3b,350,y3b+20],
  'Choose folder to store image slices:');
dlg.p3.brse1 = dlg.p3.add('edittext', [5,y3b+21,280,y3b+46],'');
dlg.p3.brse2 = dlg.p3.add('button',[285,y3b+21,360,y3b+46],'Browse');
dlg.p3.brse2.onClick = function(){
   tmp3 = getfolder();
   if (tmp3!=null){dlg.p3.brse1.text=tmp3.fsName}
    };

// --------------------- Panel 4 -----------------------------
var y4a = 5;
dlg.p4.preview = dlg.p4.add('checkbox',[5,y4a,5+170,y4a+25],
   'Generate Preview Image');
dlg.p4.preview.onClick = function() {
   tmp4b = dlg.p4.preview.value;
   dlg.p4.prevgrey.enabled = tmp4b;
   dlg.p4.prevwidth.enabled = tmp4b;
   dlg.p4.prevwidtht.enabled = tmp4b;
   dlg.p4.prevquality.enabled = tmp4b;
   dlg.p4.prevqualityt.enabled = tmp4b;
};

dlg.p4.prevgrey = dlg.p4.add('checkbox',[190,y4a,190+140,y4a+25],
  'Greyscale');

var y4b = 35;
dlg.p4.m1 = dlg.p4.add('statictext',[5,y4b+2,5+50,y4b+20+2],'Width:');
dlg.p4.prevwidth = dlg.p4.add('scrollbar',[60,y4b,310,y4b+20],2,2,G.docwidth);
dlg.p4.prevwidtht = dlg.p4.add('edittext',[320,y4b,320+40,y4b+20],'');
dlg.p4.prevwidth.onChange = function(){
  dlg.p4.prevwidtht.text = Math.round(dlg.p4.prevwidth.value)
        };
        
var y4a = 70;
dlg.p4.m2 = dlg.p4.add('statictext',[5,y4a,85,y4a+20],
 'Jpeg Quality');
dlg.p4.prevquality = dlg.p4.add('slider',[90,y4a,310,y4a+20],
    4,0,(saveweb?G.maxcompsaveforweb:G.maxcomp));
dlg.p4.prevqualityt = dlg.p4.add('edittext',[320,y4a-2,320+40,y4a+18],'');
dlg.p4.prevquality.onChange = function(){dlg.p4.prevqualityt.text =
  Math.round(dlg.p4.prevquality.value)
  };

// --------------------- Panel 5 -----------------------------
var y5a = 25;
dlg.p5.m1 = dlg.p5.add('statictext',[5,y5a+2,5+50,y5a+20+2],'Width:');
dlg.p5.width = dlg.p5.add('scrollbar',[60,y5a,310,y5a+20],1,1,1500);
dlg.p5.widtht = dlg.p5.add('edittext',[320,y5a,320+40,y5a+20],'');
dlg.p5.width.onChange = function(){
  dlg.p5.widtht.text = Math.round(dlg.p5.width.value);
  dlg.mem.text = javamemoryuse(dlg);
        };
        
var y5b = 60;
dlg.p5.m2 = dlg.p5.add('statictext',[5,y5b+2,5+50,y5b+20+2],'Height:');
dlg.p5.height = dlg.p5.add('scrollbar',[60,y5b,310,y5b+20],1,1,1500);
dlg.p5.heightt = dlg.p5.add('edittext',[320,y5b,320+40,y5b+20],'');
dlg.p5.height.onChange = function(){
  dlg.p5.heightt.text = Math.round(dlg.p5.height.value);
  dlg.mem.text = javamemoryuse(dlg);
        };

// -----------------------------------------------------------
setsliders(dlg,rows,columns,quality,custom,prefix,
        folder,preview,prevgrey,prevwidth,prevquality,
        displaywidth,displayheight,saveweb,showslices);

// -----------------------------------------------------------
var tmpa, tmpb;
var ok = 0; // Variable that says to process or cancel

// Exit, help buttons
dlg.byebye = dlg.add('button',[340,1,340+27,1+20],'X');
dlg.byebye.onClick = function(){ok = 0;dlg.close()};

dlg.help = dlg.add('button',[308,1,308+27,1+20],'?');
dlg.help.onClick = function(){help()};


// Process button
var but1y = 560,but1w = 70, but1x=10;
dlg.batch = dlg.add('button',[but1x,but1y,but1x+but1w,but1y+30],
   'Process');
dlg.batch.onClick = function(){ok = 1;dlg.close()};

// Cancel button
dlg.bye = dlg.add('button',[10+but1x+but1w,but1y,10+but1x+2*but1w,but1y+30],
   'Cancel');
dlg.bye.onClick = function(){ok = 0;dlg.close()};

// Load button
dlg.load = dlg.add('button',[40+but1x+2*but1w,but1y,40+but1x+3*but1w,but1y+30],
   'Load');
dlg.load.onClick = function(){
  tmpa = getfile();
  if (tmpa){
     aaa = readprefs(tmpa);

     rows = aaa[0];
     columns = aaa[1];
     quality = aaa[2];
     custom = aaa[3];
     prefix = aaa[4];
     folder = aaa[5];
     preview = aaa[6]; 
     prevgrey = aaa[7];
     prevwidth = aaa[8];
     prevquality = aaa[9];
     purejpeg[0]=aaa[10];
     purejpeg[1]=aaa[11];
     displaywidth = aaa[12];
     displayheight = aaa[13];
     saveweb = true; // = aaa[14];  We hardwire to be true
     showslices = aaa[15];

     setsliders(dlg,rows,columns,quality,custom,prefix,
        folder,preview,prevgrey,prevwidth,prevquality,
        displaywidth,displayheight,saveweb,showslices);

     }

  };

// Save button
dlg.save = dlg.add('button',[50+but1x+3*but1w,but1y,50+but1x+4*but1w,but1y+30],
   'Save');
dlg.save.onClick = function(){
  tmpa = savefile();
  tmpb = true;

  if (tmpa.exists){tmpb = confirm("File already exists. Overwrite?")};

  if (tmpa && tmpb){
    saveprefs(tmpa,
      Math.round(dlg.p1.rows.value),
      Math.round(dlg.p1.columns.value),
      Math.round(dlg.p2.quality.value),
      dlg.p2.custom.value,
      dlg.p3.prefix.text,
      dlg.p3.brse1.text,
      dlg.p4.preview.value,
      dlg.p4.prevgrey.value,
      Math.round(dlg.p4.prevwidth.value),
      Math.round(dlg.p4.prevquality.value),
      purejpeg[0],purejpeg[1],
      Math.round(dlg.p5.width.value),
      Math.round(dlg.p5.height.value),
      true, // dlg.p2.saveweb.value,
      showslices );
      };
  };

dlg.show();


// Erase the grid from image, but only if visible (if 'show slices' = true)
if (dlg.p1.grid.value==true){eraseallslices()};


// If the ok button was pressed, and there is no customized jpeg grid,
// then fill the entire G.qualXY array with the same value.
if (ok==1 && !dlg.p2.custom.value){
  G.qualXY = []; // clear the array
  for (y = 0;y<Math.round(dlg.p1.rows.value);y++){
     G.qualXY[y] = new Array();
     for (x = 0; x<Math.round(dlg.p1.columns.value); x++){
       G.qualXY[y][x] = Math.round(dlg.p2.quality.value);
         }
      }
    }

if (ok==1){
   saveprefs(G.fileprefs,
      Math.round(dlg.p1.rows.value),
      Math.round(dlg.p1.columns.value),
      Math.round(dlg.p2.quality.value),
      dlg.p2.custom.value,
      dlg.p3.prefix.text,
      dlg.p3.brse1.text,
      dlg.p4.preview.value,
      dlg.p4.prevgrey.value,
      Math.round(dlg.p4.prevwidth.value),
      Math.round(dlg.p4.prevquality.value),
      purejpeg[0],purejpeg[1],
      Math.round(dlg.p5.width.value),
      Math.round(dlg.p5.height.value),
      true, //  dlg.p2.saveweb.value,
      showslices
      );

}

return([ok,
        Math.round(dlg.p1.rows.value),
        Math.round(dlg.p1.columns.value),
        Math.round(dlg.p2.quality.value),
        dlg.p3.prefix.text,
        dlg.p3.brse1.text,
        dlg.p4.preview.value,
        dlg.p4.prevgrey.value,
        Math.round(dlg.p4.prevwidth.value),
        Math.round(dlg.p4.prevquality.value),
        purejpeg[0],purejpeg[1],
        Math.round(dlg.p5.width.value),
        Math.round(dlg.p5.height.value),
        true, // dlg.p2.saveweb.value,
        showslices
]);

} // end of wingui()


// Get parameters file
function getfile(){

var P = File.openDialog("Load parameters for Image Slicer",
   "txt:*.txt");
if (P){return(P)}
else {return(null)};

}

// Specify location to save Parameters file
function savefile(){
var tmp,tmp1,P,P1;
P = File.saveDialog("Save parameters for Image Slicer",
   "txt:*.txt");

if (P){
  tmp = P.fsName;  // Get full path and name of file object

  tmp1 = tmp.substring(tmp.length-4,tmp.length); // Get last 3 characters

  if (tmp1!=".txt"){ // Add .txt to end of file object
     P1 = new File(P.fsName + ".txt");
     return(P1)}
  else{return(P) }; // No need to add .txt, since it is already there
  }

else {return(null)};

}


// Set all main gui sliders, also draw the slices
function setsliders(dlg,rows,columns,quality,custom,prefix,
     folder,preview,prevgrey,prevwidth,prevquality,
     displaywidth,displayheight,saveweb,showslices){

// Saveweb is hardwired true
saveweb = true;

dlg.p1.rows.value = rows;
dlg.p1.rowst.text = Math.round(dlg.p1.rows.value);
dlg.p1.columns.value = columns;
dlg.p1.columnst.text = Math.round(dlg.p1.columns.value);

dlg.p2.custom.value = custom;
dlg.p3.prefix.text = prefix;
dlg.p3.brse1.text = folder;
dlg.p4.preview.value = preview;
dlg.p4.prevgrey.value = prevgrey;
dlg.p4.prevwidth.value = prevwidth;
dlg.p4.prevwidtht.text = Math.round(dlg.p4.prevwidth.value);


dlg.p5.width.value = displaywidth;
dlg.p5.widtht.text = Math.round(dlg.p5.width.value);
dlg.p5.height.value = displayheight;
dlg.p5.heightt.text = Math.round(dlg.p5.height.value);

// This dlg object no longer exists
// dlg.p2.saveweb.value = saveweb;

dlg.p2.quality.minvalue = 0;
dlg.p4.prevquality.minvalue = 0;
// Should we also rescale the G.qualXY array here?
if (saveweb){
      dlg.p2.quality.maxvalue = G.maxcompsaveforweb;
      dlg.p4.prevquality.maxvalue = G.maxcompsaveforweb;
      }
else{
      dlg.p2.quality.maxvalue = G.maxcomp;
      dlg.p4.prevquality.maxvalue = G.maxcomp;
      };

dlg.p2.quality.value = quality;
dlg.p2.qualityt.text = Math.round(dlg.p2.quality.value);

dlg.p4.prevquality.value = prevquality;
dlg.p4.prevqualityt.text = Math.round(dlg.p4.prevquality.value);


tmpb = dlg.p2.custom.value;
dlg.p2.quality.enabled = !tmpb;
dlg.p2.qualityt.enabled = !tmpb;
dlg.p2.grid.enabled = tmpb;
dlg.p1.rows.enabled = !tmpb;
dlg.p1.rowst.enabled = !tmpb;
dlg.p1.columns.enabled = !tmpb;
dlg.p1.columnst.enabled = !tmpb;

tmpb = dlg.p4.preview.value;
dlg.p4.prevgrey.enabled = tmpb;
dlg.p4.prevwidth.enabled = tmpb;
dlg.p4.prevwidtht.enabled = tmpb;
dlg.p4.prevquality.enabled = tmpb;
dlg.p4.prevqualityt.enabled = tmpb;

// if slices are visible (dlg.p1.grid.value = true), and we are about to 
// make them invisible (showslices = false), then erase the slices.
if (dlg.p1.grid.value && !showslices){eraseallslices()};
dlg.p1.grid.value = showslices;
creategrid(rows,columns,showslices);

dlg.mem.text = javamemoryuse(dlg);
}



// Save the currently active image using Save For Web.
// This allows the images to avoid sky banding better.
//
// jpgquality = integer from 0 to G.maxcompsaveforweb;
// filetosave = file object to save image to.
//
function Saveforweb(jpgquality,filetosave){
var OS = YourOS(); // = 1 for Mac, 0 Windows

// PS CS = version 8, PS CS2 = version 9
var version = G.PSversion;

if (version<8){return}; // < version 8, do nothing

// if file object already exists, then remove it first.
// This is to prevent Photoshop from asking permission to overwrite
// an image file.
//
// if filetosave is actually a folder that already exists, and that
// folder is not empty, then the folder will NOT be deleted.
// if the folder is empty, then it will be deleted.
if (filetosave.exists){filetosave.remove()};

// Export File if Photoshop CS2
if (version==9){        // Photoshop CS2
  var exporttype = ExportType.SAVEFORWEB;
  var sw = new ExportOptionsSaveForWeb();

  sw.blur = 0;
  sw.interlaced = false;
  sw.format = SaveDocumentType.JPEG;
  sw.includeProfile = false;
  sw.optimized = true;
  sw.quality = jpgquality;

  app.activeDocument.exportDocument(filetosave,exporttype,sw);
  return;
}

// Export File if Photoshop CS (version 8)
var id30 = charIDToTypeID( "Expr" );
var desc10 = new ActionDescriptor();
var id31 = charIDToTypeID( "Usng" );
var desc11 = new ActionDescriptor();
var id32 = charIDToTypeID( "Op  " );
var id33 = charIDToTypeID( "SWOp" );
var id34 = charIDToTypeID( "OpSa" );
desc11.putEnumerated( id32, id33, id34 );

// Save as a jpeg image
var id35 = charIDToTypeID( "Fmt " );
var id36 = charIDToTypeID( "IRFm" );
var id37 = charIDToTypeID( "JPEG" );
desc11.putEnumerated( id35, id36, id37 );

var id38 = charIDToTypeID( "Intr" );
desc11.putBoolean( id38, false );

// Jpeg image quality
var id39 = charIDToTypeID( "Qlty" );
desc11.putInteger( id39, jpgquality );

var id40 = charIDToTypeID( "QChS" );
desc11.putInteger( id40, 0 );
var id41 = charIDToTypeID( "QCUI" );
desc11.putInteger( id41, 0 );
var id42 = charIDToTypeID( "QChT" );
desc11.putBoolean( id42, false );
var id43 = charIDToTypeID( "QChV" );
desc11.putBoolean( id43, false );

// Optimize
var id44 = charIDToTypeID( "Optm" );
desc11.putBoolean( id44, true );

var id45 = charIDToTypeID( "Pass" );
desc11.putInteger( id45, 1 );

var id46 = charIDToTypeID( "blur" );
desc11.putDouble( id46, 0 );

// ICC profile
var id47 = charIDToTypeID( "EICC" );
desc11.putBoolean( id47, false );

// Matte = defines color to blend transparent pixels against.
// We turn matte off.
var id48 = charIDToTypeID( "Mtt " );
desc11.putBoolean( id48, false );
var id49 = charIDToTypeID( "MttR" );
desc11.putInteger( id49, 255 );
var id50 = charIDToTypeID( "MttG" );
desc11.putInteger( id50, 255 );
var id51 = charIDToTypeID( "MttB" );
desc11.putInteger( id51, 255 );

// Save HTML = false
var id52 = charIDToTypeID( "SHTM" );
desc11.putBoolean( id52, false );

// Save Image = true
var id53 = charIDToTypeID( "SImg" );
desc11.putBoolean( id53, true );
var id54 = charIDToTypeID( "SSSO" );
desc11.putBoolean( id54, false );
var id55 = charIDToTypeID( "SSLt" );
var list1 = new ActionList();
desc11.putList( id55, list1 );

// This might stand for Directory.
// If false, then filetosave is a file, if true
// then filetosave is a folder (in which case the name of the
// currently open document is used).
//
// For Mac, we have DIDr=true. For Windows, we have DIDr=false.
var id56 = charIDToTypeID( "DIDr" );
desc11.putBoolean( id56, (OS==1?true:false) );

// Output file object
// For Mac, we just specify the folder and then SaveForWeb will
// use the name of the document to create the file name.
// For Windows, we just use the actual file object.
var id57 = charIDToTypeID( "In  " );
desc11.putPath( id57, (OS==1? new File(filetosave.path):filetosave) );

var id58 = stringIDToTypeID( "SaveForWeb" );
desc10.putObject( id31, id58, desc11 );
executeAction( id30, desc10, DialogModes.NO );

}

// Duplicate the current document.
// docname = the name of the new document.
// The advantage of this routine over .duplicate() is that we
// can specify the name of the duplicate, whereas .duplicate cannot.
function dupdocument1(docname){
var id118 = charIDToTypeID( "Dplc" );
var desc26 = new ActionDescriptor();
var id119 = charIDToTypeID( "null" );
var ref5 = new ActionReference();
var id120 = charIDToTypeID( "Dcmn" );
var id121 = charIDToTypeID( "Ordn" );
var id122 = charIDToTypeID( "Frst" );
ref5.putEnumerated( id120, id121, id122 );
desc26.putReference( id119, ref5 );
var id123 = charIDToTypeID( "Nm  " );
desc26.putString( id123, docname );
executeAction( id118, desc26, DialogModes.NO );
}

// Duplicate the currently active layer (assumed to be the Background layer)
// and create a new document from this layer.
// The reason for doing this is so that the new document does not inherit
// the slices from the source document.
//
// The advantage of this routine over .duplicate() is that we
// can specify the name of the duplicate, whereas .duplicate cannot.
function dupdocument(docname){
var desc28 = new ActionDescriptor();
var ref8 = new ActionReference();
ref8.putClass( charIDToTypeID( "Dcmn" ) );
desc28.putReference( charIDToTypeID( "null" ), ref8 );
desc28.putString( charIDToTypeID( "Nm  " ), docname );
var ref9 = new ActionReference();

var id127 = charIDToTypeID( "Lyr " );
var id128 = charIDToTypeID( "Bckg" );  // Background layer
ref9.putProperty( id127, id128 );

desc28.putReference( charIDToTypeID( "Usng" ), ref9 );
executeAction( charIDToTypeID( "Mk  " ), desc28, DialogModes.NO );


}


// Create preview image.
// previewfile is a file object.
// width is the desired width of the preview image.
// grey is boolean, to tell if to convert to greyscale.
//
// saveweb is boolean. When true, we save the file as Save for Web.
// When false, we save as regular jpg.
//
// prevquality is the jpeg compression. When saveweb = true, then
// compression values are  (0-G.maxcompsaveforweb). When saveweb = false, then
// compression values are (0-G.maxcomp).
//
function createpreview(previewfile,width,grey,prevquality,saveweb){
// We hardwire saveweb to be true.
saveweb = true;

// Duplicate the image that is being sliced
app.activeDocument = G.docref;
dupdocument(previewfile.name);  // Duplicate the active document
var preview = app.activeDocument; // preview is the duplicate

// var preview = G.docref.duplicate(); // duplicate the image

// Resize the preview image
var height = Math.round(width * G.docheight / G.docwidth);
preview.resizeImage(width,height);

if (grey==true){preview.changeMode(ChangeMode.GRAYSCALE)};


if (!saveweb){
  // If you wish to save as a jpeg, you must convert to 8 bits first
  if (preview.bitsPerChannel==BitsPerChannelType.SIXTEEN){
     preview.bitsPerChannel=BitsPerChannelType.EIGHT};
  
  var jpegopt = new JPEGSaveOptions();
  jpegopt.quality = prevquality;
  preview.saveAs(previewfile,jpegopt,false,Extension.LOWERCASE);
}

// Should I make sure that the active document is slice before running
// saveforweb?
else{Saveforweb(prevquality,previewfile)};


preview.close(SaveOptions.DONOTSAVECHANGES);

}
  
// Create a slice from image, save, and then close.
// x0,y0,x1,y1 are the coordinates of the slice
// slicefile is a file object
function getoneslice(x0,y0,x1,y1,jpgqual,slicefile,saveweb){
// We hardwire saveweb to be true.
saveweb = true;

// Duplicate the image that is being sliced
app.activeDocument = G.docref;
dupdocument(slicefile.name);  // Duplicate the active document
var slice = app.activeDocument; // slice is the duplicate

// var slice = G.docref.duplicate(); // duplicate the image


slice.crop([x0,y0,x1,y1],0);


if (!saveweb){
  // If you wish to save as a jpeg, you must convert to 8 bits first
  if (slice.bitsPerChannel==BitsPerChannelType.SIXTEEN){
      slice.bitsPerChannel=BitsPerChannelType.EIGHT};

  var jpegopt = new JPEGSaveOptions();
  jpegopt.quality = jpgqual;
  slice.saveAs(slicefile,jpegopt,false,Extension.LOWERCASE);
}

// Should I make sure that the active document is slice before running
// saveforweb?
else{Saveforweb(jpgqual,slicefile)};


slice.close(SaveOptions.DONOTSAVECHANGES);

}

// Verify the existence of the folder.
// folder = string.
// I still no good way of detecting whether a file with name = folder
// exists.
function verifyfolder(folder){
var tmp = folder;

// Check if folder is blank. If it is, then return false
if (tmp.replace(/\s/g,"")==""){
  alert("Error: No output folder was specified.\nProgram aborted.");
  return false};

var f1 = new Folder(folder);
if (!f1.exists){
  alert("Error: Specified output folder does \nnot exist. Program aborted.");
  return false
  };
  
return true;

}

  
// Create grid of image slices and save them in a folder.
// We need to verify that the folder exists first before trying to 
// create the slices, however.
function allslices(aa){
if (aa[0]==0){return};

var rows = aa[1], columns = aa[2], jpgqual = aa[3], 
  prefix = aa[4], folder = aa[5], 
  preview = aa[6], // boolean
  prevgrey = aa[7], // boolean
  prevwidth = aa[8], // width of preview image
  prevquality = aa[9], // jpeg quality of preview image
  pure = aa[10], // boolean
  purejpegexe = aa[11], // purejpeg location
  displaywidth = aa[12],
  displayheight = aa[13],
  saveweb = true, // aa[14],  Save for Web? (Boolean)
  slicefile;
  
if (!verifyfolder(folder)){return};

// Check if prefix is blank. If it is, then use a default value
if (prefix.replace(/\s/g,"")==""){prefix = "slice"};


// Create preview image
if (preview==true){
  var previewfile = new File(folder + "/" + previewname(prefix));
  createpreview(previewfile,prevwidth,prevgrey,prevquality,saveweb);
 }

// Now loop thru all the slices
for (var x=0;x<columns;x++){
  for (var y=0;y<rows;y++){
    slicefile = new File(folder + "/" + tilename(prefix,x,y));

// Note that the jpeg quality for each slice is stored in a global array
    jpgqual = G.qualXY[y][x];
//alert("array lengths = " + G.gridX.length + "," + G.gridY.length)
    getoneslice(G.gridX[x],G.gridY[y],
       G.gridX[x+1],G.gridY[y+1],jpgqual,slicefile,saveweb);
  }
}

createhtml(rows,columns,prefix,folder,preview,displaywidth,displayheight);

// Create ptvref file and html
createptvref(rows,columns,prefix,folder,preview);
createhtmlptv(false,rows,columns,prefix,folder,preview,displaywidth,
     displayheight);

// Create ptref file and html
createptv(rows,columns,prefix,folder,preview,prevwidth);
createhtmlptv(true,rows,columns,prefix,folder,preview,displaywidth,
     displayheight);

}

// Create html (xhtml) code for the roi's
function createhtml(rows,columns,prefix,folder,
    preview,displaywidth,displayheight){
var a="";
appletcode = new File(folder + "/" + prefix + ".html");

a +="<html>\n<head></head>\n<body>\n\n";

a += "<div align=\"center\">\n";
a += "<applet archive=\"ptviewer.jar\" code=\"ptviewer.class\" " + 
     "id=\"ptviewer\"\n" +
     "    mayscript=\"true\" width=\"" + displaywidth + "\" height=\"" +
     displayheight + "\">\n";

if (preview==true){
  a += "<param name=\"file\" value=\"" + previewname(prefix) + "\" />\n";
};

a += "<param name=\"cursor\" value=\"move\" />\n";
a += "<param name=\"showToolbar\" value=\"true\" />\n";

a += "\n";

a += "<param name=\"fov\" value=\"90\" />\n";
a += "<param name=\"fovmin\" value=\"40\" />\n";
a += "<param name=\"fovmax\" value=\"130\" />\n";
a += "<param name=\"pan\" value=\"0\" />\n";
a += "<param name=\"tilt\" value=\"0\" />\n";
a += "<param name=\"tiltmin\" value=\"-90\" />\n";
a += "<param name=\"tiltmax\" value=\"90\" />\n";

a += "\n";

a += "<param name=\"dynLoadROIs\" value=\"true\" />\n";

for (var y=0;y<rows;y++){
  for (var x=0;x<columns;x++){
  a += "<param name=\"roi" + (columns*y + x) +  "\" value=\" i\'" + 
      tilename(prefix,x,y) + "\' x" + G.gridX[x] + " y" + G.gridY[y] +
      " w" + (G.gridX[x+1] - G.gridX[x]) +
      " h" + (G.gridY[y+1] - G.gridY[y]) + "\" />\n";
   }
 }

a += "<param name=\"pwidth\" value=\"" + G.docwidth + "\" />\n" +
     "<param name=\"pheight\" value=\"" + G.docheight + "\" />\n";
a += "</applet>\n</div>\n\n";

a += "<form action=\"\" method=\"get\" name=\"posit\">\n" +
  "<button name=\"updateposition\" type=\"button\" " +
  "onClick=\"displayposition(this.form)\">position</button>\n" +
  "&nbsp;Pan:<input type=\"text\" name=\"pan\" value=\"0\" size=\"4\" />\n" +
  "&nbsp;Tilt:<input type=\"text\" name=\"tilt\" value=\"0\" size=\"4\" />\n" +
  "&nbsp;Fov:<input type=\"text\" name=\"fov\" value=\"70\" size=\"4\" />\n" +
  "</form>\n\n";

a += "<script type=\"text/javascript\">\n" +
  "<!--\n" +
  "function displayposition(f){\n" +
  "if (document.getElementById && document.getElementById(\"ptviewer\")){\n" +
  " var  p = document.getElementById(\"ptviewer\").pan();\n" +
  " f.pan.value = Math.round(p*100)/100;\n" +
  " var  t = document.getElementById(\"ptviewer\").tilt();\n" +
  " f.tilt.value = Math.round(t*100)/100;\n" +
  " var  foview = document.getElementById(\"ptviewer\").fov();\n" +
  " f.fov.value = Math.round(foview*100)/100;\n" +
  "  }\n" +
  " }\n" +
  "/" + "/-->\n" +
  "</script>\n\n";

a +="</body>\n</html>\n";

writeall(appletcode,a);
}


// Create html (xhtml) code for the ptvref/ptv file and the roi's.
// ptv = true, create html for ptv file
// ptv = false, create html for ptvref file
function createhtmlptv(ptv,rows,columns,prefix,folder,
    preview,displaywidth,displayheight){
var a="";

appletcode = new File(folder + "/" + prefix +
     (ptv==true?"ptv.html":"ptvref.html") );

a +="<html>\n<head></head>\n<body>\n\n";

a += "<div align=\"center\">\n";
a += "<applet archive=\"ptviewer.jar\" code=\"ptviewer.class\" " +
     "width=\"" + displaywidth + "\" height=\"" +
     displayheight + "\">\n";

a += "<param name=\"file\" value=\"" + prefix +
     (ptv==true?".ptv":".ptvref") + "\" />\n";

a += "<param name=\"cursor\" value=\"move\" />\n";
a += "<param name=\"showToolbar\" value=\"true\" />\n";

a += "\n";

a += "<param name=\"fov\" value=\"90\" />\n";
a += "<param name=\"fovmin\" value=\"40\" />\n";
a += "<param name=\"fovmax\" value=\"130\" />\n";
a += "<param name=\"pan\" value=\"0\" />\n";
a += "<param name=\"tilt\" value=\"0\" />\n";
a += "<param name=\"tiltmin\" value=\"-90\" />\n";
a += "<param name=\"tiltmax\" value=\"90\" />\n";

a += "\n";

a += "</applet>\n</div>\n\n";

a +="</body>\n</html>\n";

writeall(appletcode,a);
}


// Create ptvref file
function createptvref(rows,columns,prefix,folder,preview){
var a="";
appletcode = new File(folder + "/" + prefix + ".ptvref");

// Preview image
if (preview==true){
  a += previewname(prefix) + "\n";
}
else{a += "\n"};

a += G.docwidth +  "\n" +
     G.docheight + "\n" +
     (rows*columns) + "\n";
     
for (var y=0;y<rows;y++){
  for (var x=0;x<columns;x++){
  a += "i\'" + tilename(prefix,x,y) + "\' x" +
       G.gridX[x] + " y" + G.gridY[y] +
      " w" + (G.gridX[x+1] - G.gridX[x]) +
      " h" + (G.gridY[y+1] - G.gridY[y]) +  "\n";
   }
 };

writeall(appletcode,a);

}

// Create ptv file. A ptv file is nothing more than a single container
// for all the image slices.
// In order to create the ptv file, we need to create a BINARY encoded
// file.
function createptv(rows,columns,prefix,folder,preview,prevwidth){
if (!File.isEncodingAvailable("BINARY")){return(false)};

// The header is the initial part of the ptv file
var header = "";
header += "5054"; // Signature
header += decimal2hex(1,2); // Version number = 1
header += decimal2hex(G.docwidth,4); // Width of pano, 4 bytes
header += decimal2hex(G.docheight,4); // Height of pano, 4 bytes

var sizeofdirheader = 6, sizeofdirentry = 28;
header += decimal2hex(sizeofdirheader,4); // Size of the directory header
header += decimal2hex(sizeofdirentry,4); // Size of a directory entry

var offset = 80;  // offset is the byte offset in the ptv file
header += decimal2hex(offset,4); // Offset of first byte of directory
while (header.length < offset*2){header += "00"}; // Unused bytes

// The directory comes after the header in the ptv file
var directory="";
directory += decimal2hex(rows,2);  // header in directory
directory += decimal2hex(columns,2);  // header in directory
directory += decimal2hex((preview==true?1:0),2); // header in directory

// Calculate offset of first image
offset += sizeofdirheader +
    (rows*columns + (preview==true?1:0))*sizeofdirentry;


var tmpfile;

var dummybytes = 1 // Number of dummy bytes between images
var dummy,i;

// bufferfile is a file that temporarily holds all the images,
// including the preview image. As bufferfile increases in size each
// time we add an image to it, we can determine the change in the file pointer,
// and therefore know the exact length of each individual roi image.
var bufferfile = new File(folder + "/" + "buffer");
bufferfile.length=0;    // Set the length=0 BEFORE open()
bufferfile.open("w",G.MACtype,G.MACcreator);
bufferfile.encoding = "BINARY";

// We use bufferfile, and pointer1,pointer2 to find the file size
// of each individual image slice.
var pointer1,pointer2; // Used to find size of files

// Create directory entry for preview image
if (preview==true){
  directory += decimal2hex(0,2); // Row 0
  directory += decimal2hex(0,2); // Column 0
  directory += decimal2hex(prevwidth,4); // Width of preview image
  directory += decimal2hex(
     Math.round(prevwidth * G.docheight / G.docwidth),4); // Height
  directory += "00000000"; // x position
  directory += "00000000"; // y position
  directory += decimal2hex(offset,4); // Offset of preview image

 
  tmpfile = new File(folder + "/" + previewname(prefix));
  tmpfile.open("r",G.MACtype,G.MACcreator);
  tmpfile.encoding = "BINARY";
  pointer1 = bufferfile.tell();
  bufferfile.write(tmpfile.read()); // Add preview to the bufferfile
  pointer2 = bufferfile.tell();
  directory += decimal2hex(pointer2 - pointer1,4); // size of image

  tmpfile.close();

  dummy = "";
  for (i=0;i < dummybytes; i++){dummy += "00"};
  writehexfile(bufferfile,dummy);  // Add dummy bytes
  
  offset += pointer2 - pointer1 + dummybytes; // We put dummy byte in here
};

// Create directory entries for each image slice
for (var y=0;y<rows;y++){
  for (var x=0;x<columns;x++){
     directory += decimal2hex(y,2); // Row y
     directory += decimal2hex(x,2); // Column x
     directory += decimal2hex((G.gridX[x+1] - G.gridX[x]),4); // Slice width
     directory += decimal2hex((G.gridY[y+1] - G.gridY[y]),4); // Slice height
     directory += decimal2hex(G.gridX[x],4); // x position in pano
     directory += decimal2hex(G.gridY[y],4); // y position in pano
     
     directory += decimal2hex(offset,4); // Offset of slice image
     
     tmpfile = new File(folder + "/" + tilename(prefix,x,y));
     tmpfile.open("r",G.MACtype,G.MACcreator);
     tmpfile.encoding = "BINARY";
     pointer1 = bufferfile.tell();
     bufferfile.write(tmpfile.read()); // Add roi image to the bufferfile
     pointer2 = bufferfile.tell();
     directory += decimal2hex(pointer2 - pointer1,4); // size of image

     tmpfile.close();

     dummy = "";
     for (i=0;i < dummybytes; i++){dummy += "00"};
     writehexfile(bufferfile,dummy); // Add dummy bytes
     
     offset += pointer2 - pointer1 + dummybytes;
      }
 };

bufferfile.close();

// Now we create the ptv file
ptv = new File(folder + "/" + prefix + ".ptv");

ptv.open("w",G.MACtype,G.MACcreator);
ptv.encoding = "BINARY";
writehexfile(ptv,header); // Write header
writehexfile(ptv,directory); // Write directory

bufferfile.open("r",G.MACtype,G.MACcreator);
bufferfile.encoding = "BINARY";

ptv.write(bufferfile.read());  // Copy all the images slices into ptv file

bufferfile.close();
ptv.close();

// Delete the bufferfile
bufferfile.remove();
}


// Write the hexstring 'hex' to file f, which is a BINARY file.
// hex is a string containing hex bytes in the form of "FFECAB01" etc.
// Each pair of digits in the string forms a byte (8 bits).
// hex must have an even string length.
//
// To generate a true hex containing string, we use unescape("%FF");
// Note: do not use "\u00AA" or "\xAA" to create hex strings, because
// "\u" + "00AA" does not equal to "\u00AA". You cannot manipulate this.
function writehexfile(f,hex){
var a,h;
for (var i=0;i<hex.length;i+=2){
  a = hex.substring(i,i+2); if (a.length==1){a += "0"}
  h = unescape("%" + a);
  f.write(h);
}

}


// Recursive function that converts decimal to hexadecimal.
// The LSB is the leftmost character, the MSB is the rightmost character.
function decimaltohex(num){
var s = ["0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"];
var k0 = num % 16;
var k1 = Math.round((num-k0)/16);

if (k1>0) {return(decimaltohex(k1) + s[k0])}
else {return(s[k0])};
}

// Convert decimal num to hexadecimal.
// numbytes is the number of bytes to return (1 bytes = 2 hex digits).
// Least significant byte is the left most byte.
//
// Should we look for the possibility that numbytes is too small to 
// contain the hex result for num?
function decimal2hex(num,numbytes){
var a = decimaltohex(num); // Convert to hexadecimal

var tmp = a.length
for (var i=0; i < numbytes*2-tmp; i++){a = "0" + a}


if (a.length % 2 != 0){a = "0" + a};

// Now, we reverse the order of the bytes, so that the least sig. byte is
// on the left, and the most significant byte is on the right.
var b = "";
for (i=0; i<a.length; i+=2){b = a.substring(i,i+2) + b};

return(b);
}



// The name of each slice (with the .jpg extension)
// prefix is a string,
// x and y are integers
function tilename(prefix,x,y){
return(prefix + x.toString() + y.toString() + ".jpg")
};

// The name of the preview image (with the .jpg extension)
function previewname(prefix){
return(prefix + "prev.jpg")
};


// Write string 'a' into file 'f'
function writeall(f,a){
f.length=0;    // Set the length=0 BEFORE open()
f.open("w",G.MACtype,G.MACcreator);
f.write(a);
f.close();
a=null;
}

// Save preferences to a file
//  a = rows,
//  b = columns,
//  c = jpg quality (0 - maxcompression)
//  d = custom quality (boolean)
//  e = prefix,
//  f = folder path for output
//  g = preview image(boolean)
//  h = greyscale for preview (boolean)
//  i = preview width
//  j = preview jpeg quality (0 - maxcompression)
//  k = Use purejpeg.exe to remove exif (boolean)
//  m = path to purejpeg.exe
//  n = display window width
//  p = display window height
//  q = save for web (boolean)
//  r = show slices (boolean)
//
// prefs is the file to save the preferences to.
function saveprefs(prefs,a,b,c,d,e,f,g,h,i,j,k,m,n,p,q,r){
var z;
z = a + "\n" + b + "\n" + c + "\n" + (d?1:0) + "\n" + e + "\n" +
   f + "\n";

// Store the array on a single line
var columns = b, rows = a;
z += "[";
for (var y=0;y<rows;y++){
  z += (y!=0?",":"") + "[";
  for (var x=0;x<columns;x++){
     z += G.qualXY[y][x] + (x<columns-1?",":"");
   }
  z += "]";
 }
z += "]\n";

z += (g?1:0) + "\n" + (h?1:0) + "\n" + i + "\n" + j + "\n" +
  (k?1:0) + "\n" + m + "\n" + n + "\n" + p + "\n" + (q?1:0) + "\n" +
  (r?1:0) + "\n";

writeall(prefs,z);

}

// Read Preferences file.
// q specifies the location of the preferences file.
function readprefs(q){
var rows = G.rowsdef,
    columns = G.columnsdef,
    quality = G.qualitydef,
    custom = G.customdef,
    prefix = G.prefixdef,
    folder = G.folderdef,
    preview = G.previewdef,
    prevgrey = G.prevgreydef,
    prevwidth = G.prevwidthdef,
    prevquality = G.prevqualdef;
    pure = G.puredef;               // Use purejpeg? (boolean)
    purelocation = G.purelocdef;    // Path to purejpeg
    displaywidth = G.diswidthdef;   // Display window width
    displayheight = G.disheightdef; // Display window height
    saveweb = G.savewebdef;      // Save for web? (boolean)
    showslices = G.showslicesdef;

if (q.exists){ // Does file exist? Yes
     q.open("r",G.MACtype,G.MACcreator); // read mode

// We initialize the Line[] array to default values. Each array element
// contains a string (not a number, not a boolean).
// I use 'decimal(xxx)' to make sure that xxx is a
// number, and then dot(decimal(xxx)) to make sure its a string.
var Line=[dot(decimal(G.rowsdef)),
          dot(decimal(G.columnsdef)),
          dot(decimal(G.qualitydef)),
          G.customdef?"1":"0",
          G.prefixdef,
          G.folderdef,
          "[]",    // This line holds G.qualXY array
          G.previewdef?"1":"0",
          G.prevgreydef?"1":"0",
          dot(decimal(G.prevwidthdef)),
          dot(decimal(G.prevqualdef)),
          G.puredef?"1":"0",
          G.purelocdef,
          dot(decimal(G.diswidthdef)),
          dot(decimal(G.disheightdef)),
          G.savewebdef?"1":"0",
          G.showslicesdef?"1":"0"
        ];

// Read in Prefs from preferences file
for (var i=0;i<Line.length;i++){
  if (!q.eof){Line[i]=q.readln()}
    }

q.close();

// Note: The statement 'decimal(Line)' assumes that Line
// is either a number or a string. You should not send a Boolean as input
// to decimal().


// Get rows value
if ((Line[0]!=null)&&(Line[0].length>0)){
  rows = decimal(Line[0])};

// Get columns value
if ((Line[1]!=null)&&(Line[1].length>0)){
  columns = decimal(Line[1])};

// Get quality value
if ((Line[2]!=null)&&(Line[2].length>0)){
  quality = decimal(Line[2])};

// Get custom (boolean)
if ((Line[3]!=null)&&(Line[3].length>0)){
  custom = (decimal(Line[3])==0)?false:true};

// Get prefix value (string)
if ((Line[4]!=null)&&(Line[4].length>0)){
  prefix = Line[4]};

// Get folder value (string)
if ((Line[5]!=null)&&(Line[5].length>0)){
  folder = Line[5]};

// Line[6] contains the G.qualXY array
// Read in the array here
var x,y,q1;
G.qualXY = [];
if (!custom){
  for (y=0;y<rows;y++){
     G.qualXY[y] = new Array();
     for (x=0;x<rows;x++){G.qualXY[y][x]= quality};
     }
}
else{
   eval("G.qualXY = " + Line[6]);
};


// Get preview (boolean)
if ((Line[7]!=null)&&(Line[7].length>0)){
  preview = (decimal(Line[7])==0)?false:true};

// Get prevgrey (boolean)
if ((Line[8]!=null)&&(Line[8].length>0)){
  prevgrey = (decimal(Line[8])==0)?false:true};

// Get prevwidth value
if ((Line[9]!=null)&&(Line[9].length>0)){
  prevwidth = decimal(Line[9])};

// Get prevquality value
if ((Line[10]!=null)&&(Line[10].length>0)){
  prevquality = decimal(Line[10])};

// Get pure (boolean)
if ((Line[11]!=null)&&(Line[11].length>0)){
  pure = (decimal(Line[11])==0)?false:true};

// Get purelocation value (string)
if ((Line[12]!=null)&&(Line[12].length>0)){
  purelocation = Line[12]};
  
// Get displaywidth value
if ((Line[13]!=null)&&(Line[13].length>0)){
  displaywidth = decimal(Line[13])};
  
// Get displayheight value
if ((Line[14]!=null)&&(Line[14].length>0)){
  displayheight = decimal(Line[14])};

// Get saveweb (boolean)
if ((Line[15]!=null)&&(Line[15].length>0)){
  saveweb = (decimal(Line[15])==0)?false:true};
  
// Get showslices (boolean)
if ((Line[16]!=null)&&(Line[16].length>0)){
  showslices = (decimal(Line[16])==0)?false:true};

};  // end of if (q.exists) statement

Line=null;

return [rows,columns,quality,custom,
        prefix,folder,
        preview,prevgrey,prevwidth,prevquality,
        pure,purelocation,
        displaywidth,displayheight,
        saveweb,showslices];

}

// Input is a number, with either '.' or ',' in it. We return a string.
// We convert all nums with ',' to '.'
// For example, 0,0 -> 0.0  0,5 -> 0.5
// This is in case international PS versions use ',' but we want to
// print out '.'
// This script assumes that PTOptimizer uses decimal points, even if the
// local number systems uses commas.
function dot(a){
a = a.toString();
var rr = /,/g;  // Regular expression. 'g' = global search and replace of ','
return(a.replace(rr , "."));
}

// Here we try to calculate the memory used by Java to
// display a panorama.
function javamemoryuse(dlg){
var displaywidth = Math.round(dlg.p5.width.value);
var displayheight = Math.round(dlg.p5.height.value);
var rows = Math.round(dlg.p1.rows.value);
var columns = Math.round(dlg.p1.columns.value);

var memory = (4 * G.docwidth * G.docheight) +
   4 * G.docwidth * G.docheight/(rows * columns);
memory += displaywidth * displayheight * 5;

var result = memory/1e6;
result = Math.round(result*100)/100;

//return('Memory:')
//return('Memory: ' + result + " x 10^6 bytes");
return ("Image: " + G.docwidth + " x " + G.docheight);
}


// Draw a grid on the image based on the # columns and rows.
// A slice is then given by G.gridX[i],G.gridY[j] to G.gridX[i+1],G.gridY[j+1]
//
// showslices = boolean. Tells if the slices are currently visible.
function creategrid(rows,columns,showslices){
// If the slices are visible, then erase them first.
// If they are invisible, then there is nothing to erase.
if (showslices==true){eraseallslices()};

G.gridX = []; G.gridY = [];

for (var i = 0; i<=columns; i++){
 G.gridX[i]=Math.round(i*G.docwidth/columns);
}

for (i = 0; i<=rows; i++){
 G.gridY[i]=Math.round(i*G.docheight/rows);
                               };

if (showslices){ // if showslices=true, then draw the slices
  //divideslice(rows,columns);
  for (var y=0;y<G.gridY.length-1;y++){
    for (var x=0;x<G.gridX.length-1;x++){
       drawslice(G.gridX[x],G.gridY[y],G.gridX[x+1],G.gridY[y+1]);
    }};
 }

}

// Erase all slices, but only if more than one slice has been drawn.
// Return false is slices not visible, and thus cannot be erased.
// Return false if no slices exist.
// Return true if slices successfully erased.
function eraseallslices(){
// Photoshop CS2 cannot erase all the slices using
// script listener code. This looks like a CS2 bug.
if (G.PSversion==9){return false};

// If there is only one slice, then that one slice covers the entire
// image. Hence, there is nothing to erase, so return.
if (G.gridX.length <= 2 && G.gridY.length <= 2){return false};

var id1105 = charIDToTypeID( "Dlt " );
var desc243 = new ActionDescriptor();
var id1106 = charIDToTypeID( "null" );
var ref140 = new ActionReference();
var id1107 = stringIDToTypeID( "slice" );
var id1108 = charIDToTypeID( "Ordn" );
var id1109 = charIDToTypeID( "Al  " );
ref140.putEnumerated( id1107, id1108, id1109 );
desc243.putReference( id1106, ref140 );

try{
  executeAction( id1105, desc243, DialogModes.NO );
  return true
   }
catch(e){return false}


}

// Divide image into evenly spaced slices
function divideslice(rows,columns){

var id132 = stringIDToTypeID( "divide" );
var desc32 = new ActionDescriptor();
var id133 = charIDToTypeID( "null" );
var ref12 = new ActionReference();
var id134 = stringIDToTypeID( "slice" );
var id135 = charIDToTypeID( "Ordn" );
var id136 = charIDToTypeID( "Trgt" );
ref12.putEnumerated( id134, id135, id136 );
desc32.putReference( id133, ref12 );
var id137 = stringIDToTypeID( "slicesDown" );
desc32.putInteger( id137, 6 );
var id138 = stringIDToTypeID( "slicesAcross" );
desc32.putInteger( id138, 11 );
executeAction( id132, desc32, DialogModes.NO );

}


// Draw a slice in the image
function drawslice(x0,y0,x1,y1){
// There is no point in drawing slices in PS CS2, because we have
// trouble erasing them.
if (G.PSversion==9){return false}; // PS CS2

var id80 = charIDToTypeID( "Mk  " );
var desc17 = new ActionDescriptor();
var id81 = charIDToTypeID( "null" );
var ref8 = new ActionReference();
var id82 = stringIDToTypeID( "slice" );
ref8.putClass( id82 );
desc17.putReference( id81, ref8 );
var id83 = charIDToTypeID( "Usng" );
var desc18 = new ActionDescriptor();
var id84 = charIDToTypeID( "Type" );
var id85 = stringIDToTypeID( "sliceType" );
var id86 = stringIDToTypeID( "user" );
desc18.putEnumerated( id84, id85, id86 );
var id87 = charIDToTypeID( "At  " );
var desc19 = new ActionDescriptor();

var id88 = charIDToTypeID( "Top " );
var id89 = charIDToTypeID( "#Pxl" );
desc19.putUnitDouble( id88, id89, y0 );

var id90 = charIDToTypeID( "Left" );
var id91 = charIDToTypeID( "#Pxl" );
desc19.putUnitDouble( id90, id91, x0 );

var id92 = charIDToTypeID( "Btom" );
var id93 = charIDToTypeID( "#Pxl" );
desc19.putUnitDouble( id92, id93, y1 );

var id94 = charIDToTypeID( "Rght" );
var id95 = charIDToTypeID( "#Pxl" );
desc19.putUnitDouble( id94, id95, x1 );

var id96 = charIDToTypeID( "Rctn" );
desc18.putObject( id87, id96, desc19 );
var id97 = stringIDToTypeID( "slice" );
desc17.putObject( id83, id97, desc18 );
executeAction( id80, desc17, DialogModes.NO );

}


function main(){
if (app.documents.length==0){
   alert("Error! No image documents are open.");
   return(false);
   };
   
if (PSversion()<8){
   alert("Error: This script only runs on Photoshop\n" +
   "version 8 or higher.");
   return(false);
   };

if (app.activeDocument.layers.length==0){return}

// Check that only 1 layer in the active doc
if (app.activeDocument.layers.length>1){
    if (confirm("This script requires that only one layer\n" +
    "is in the image. Permission to flatten the image?") ){
       app.activeDocument.flatten()}
    else{return};
}

// Make sure that the only layer is the background layer
if (!app.activeDocument.activeLayer.isBackgroundLayer){
    if (confirm("This script requires the image to consist of\n" +
    "a Background layer. Permission to flatten the image?") ){
        app.activeDocument.flatten()}
    else{return};
}


savestate(); // Save photoshop state

globalparams();

allslices(wingui());

restorestate(); // Restore photoshop state
}


// ---------------------------------------------------------------------------

var G = new Object(); // Global object

main();



